// cheapls-direntries -- cheapls, but using getdirentries()

#import <sys/types.h>	// for random type definition
#import <sys/dirent.h>	// for struct dirent
#import <dirent.h>	// for getdirentries()
#import <stdlib.h>	// for EXIT_SUCCESS
#import <stdio.h>	// for printf
#import <errno.h>	// for errno
#import <string.h>	// for strerror
#import <fcntl.h>	// for O_RDONLY
#import <sys/stat.h>	// for struct statbuf and stat()

/* compile with
cc -g -o cheapls-direntries cheapls-direntries.m
*/

int main (int argc, char *argv[])
{
    // sanity check the program argument first
    if (argc != 2) {
	fprintf (stderr, "usage:  %s /path/to/directory\n", argv[0]);
	exit (EXIT_FAILURE);
    }

    int fd;
    fd = open (argv[1], O_RDONLY);

    if (fd == -1) {
        fprintf (stderr, "could not open directory '%s'\n", argv[1]);
        fprintf (stderr, "error: %d/%s\n", errno, strerror(errno));
        exit (EXIT_FAILURE);
    }

    // make sure it is actually a directory
    // also gives us the blocksize for the getdirentries call
    struct stat statbuf;
    int result;
    result = fstat (fd, &statbuf);

    if (result == -1) {
        fprintf (stderr, "could not stat directory '%s'\n", argv[1]);
        fprintf (stderr, "error: %d/%s\n", errno, strerror(errno));
        exit (EXIT_FAILURE);
    }

    if (! (statbuf.st_mode & S_IFDIR)) {
        fprintf (stderr, "%s is not a directory\n", argv[1]);
        exit (EXIT_FAILURE);
    }

#define BLOCK_COUNT 10
    char *buffer;
    buffer = malloc (statbuf.st_blksize * BLOCK_COUNT);
    long base = 0; // all of these are belong to us

    // get first batch of directory entries
    result = getdirentries (fd, buffer, 
                            statbuf.st_blksize * BLOCK_COUNT, &base);
    int position = 1;

    while (result > 0) {
        // walk through the returned buffer
        char *scan, *stop;
        scan = buffer;
        stop = scan + result;

        while (scan < stop) {
            struct dirent *entry = (struct dirent *) scan;
            
            printf ("%3ld: %s\n", position, entry->d_name);

            scan += entry->d_reclen;
            position++;
        }

        // get the next batch (or zero when we run out)
        result = getdirentries (fd, buffer, 
                                statbuf.st_blksize * BLOCK_COUNT, &base);
    }
    free (buffer);

    if (result == -1) {
        fprintf (stderr, "error using getdirentries: %d/%s\n",
                 errno, strerror(errno));
        exit (EXIT_FAILURE);
    }

    close (fd);

    return (EXIT_SUCCESS);

} // main

